/*  The Blue Mango Quest
 *  Copyright (c) Clment 'phneutre' Bourdarias (code)
 *                   email: phneutre@users.sourceforge.net
 *                Guillaume 'GuBuG' Burlet (graphics)
 *                   email: gubug@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef WIN32
#include <windows.h>
#endif

#include <GL/gl.h>
#include <GL/glu.h>
#include <math.h>
#include <stdlib.h>
#include <iostream.h>
#include <SDL/SDL.h>

#include "timers.h"
#include "world_geometry.h"
#include "hut.h"
#include "bonus.h"
#include "mango.h"
#include "world_building.h"
#include "texture.h"
#include "world_geometry.h"
#include "sector.h"
#include "map.h"

#define MAX_LIVES 5

const char hasardTable[20] = {
  1,1,1,1,0/*onde*/,1,1,1,1,0,1,1,1,1,1,1,1,1,1,0
};

const char *textureBonus[NUM_BONUS] = {

  SHXMAN_DATA "textures/bonus/shmixgomme.png",
  SHXMAN_DATA "textures/bonus/vie.png",
  SHXMAN_DATA "textures/bonus/bouclier.png",
  SHXMAN_DATA "textures/bonus/armaggedon.png",
  SHXMAN_DATA "textures/bonus/onde.png",
  SHXMAN_DATA "textures/bonus/stoptemps.png",
  SHXMAN_DATA "textures/bonus/ultrapoint.png",
  SHXMAN_DATA "textures/bonus/glacon.png",
  SHXMAN_DATA "textures/bonus/rapide.png",
  SHXMAN_DATA "textures/bonus/coupdebol.png",
  
  SHXMAN_DATA "textures/bonus/nuit.png",
  SHXMAN_DATA "textures/bonus/perdbonus.png",
  SHXMAN_DATA "textures/bonus/inverttouches.png",
  SHXMAN_DATA "textures/bonus/plafond.png",
  SHXMAN_DATA "textures/bonus/fog.png",
  SHXMAN_DATA "textures/bonus/malusmap.png",
  SHXMAN_DATA "textures/bonus/boitesnegatives.png",
  SHXMAN_DATA "textures/bonus/freezeshx.png", 
  SHXMAN_DATA "textures/bonus/lent.png",
  SHXMAN_DATA "textures/bonus/hasard.png"
};

const char *textureDituboite = SHXMAN_DATA "textures/bonus/dituboite.png";

extern game_data_t *world;
extern player_t *player;

int whichBonusActive()
{
  for (int i=0; i<NUM_BONUS; i++)
    {
      if ((i!=E_SHMIXGOMME)&&(world->tab_bonus[i]->actif)) 
	return i;
    }
  return -1;
}

void execute_bonus(int type)
{
  int parametre = get_bonus_param(player->square);
  world->map[player->square].bonus = -2;
  if (!(desactive_bonus(player->mySector, player->square)))
    fprintf(stderr,"Error: can't find bonus on square %d\n",
	    player->square);
  world->tab_bonus[type]->agir(parametre);
}

void stock_bonus(int number)
{
  world->map[player->square].bonus = -2;
  if (!(desactive_bonus(player->mySector, player->square)))
    fprintf(stderr,"Error: can't find bonus on square %d\n",
	    player->square);
  player->stockBonus[number]=1;
  player->paramStocks[number]=get_bonus_param(player->square);
}

void locate_dituboites()
{
  for (int i = 0; i < world->size_square; i++)
  {
    if ((world->map[i].tex_sol > -1) && (world->map[i].bonus == -1)
        && (world->map[i].special == -1))
      {
	world->map[i].is_ditugomme = 1;
	world->num_dituboites++;
      }
    else
      world->map[i].is_ditugomme = 0;
  }

}

BONUS::~BONUS()
{

}

void BONUS::construit(GLint listeType)
{
  printf("ICI\n");
  
}

void BONUS::agir(int parametre)
{

}

void BONUS::annuleEffets()
{

}


void BONUS::affiche(float x, float y, float z, char with_rotat)
{
  glPushMatrix();
  glEnable(GL_TEXTURE_2D);

  glRotatef(-90.0f, 1.0f, 0.0f, 0.0f);
  glTranslatef(x, -z, y);

  if (with_rotat)
    glRotatef(world->bonus_rotat, 0.0f, 0.0f, 1.0f);

  glBindTexture(GL_TEXTURE_2D, texture);

  glCallList(liste);
  glDisable(GL_TEXTURE_2D);
  glPopMatrix();
}


char BONUS::updateTimer()
{
  if (timer->update_temps()) {
    printf("Bonus's effect is finished\n");
    annuleEffets();
    actif=0;
    return 0;
  }

  else return 1;
}

void BONUS::cancelAction()
{
  printf("The bonus is no longer active (by cancel)\n");
  annuleEffets();
  actif=0;
}

void BONUS::setUpTimer(int parametre)
{
  timer->resetTemps();
  timer->setAlarm(parametre);
}

void DITUBOITE::agir(int parametre)
{
  player->score+=valeur;
}

void DITUBOITE::construit(GLint typeListe)
{
  liste = typeListe;
  LoadTexture(textureDituboite, &texture);
}

void DITUBOITE::annuleEffets()
{
}


void B_VIE::agir(int parametre)
{
  if (player->num_lives < MAX_LIVES) player->num_lives++;
  player->score += 20;
}

void B_VIE::construit(GLint typeListe)
{ liste = typeListe;
  LoadTexture(textureBonus[E_VIE], &texture);
}

void B_VIE::annuleEffets()
{
}

void B_HASARD::agir(int parametre)
{
  int theChoice=0, i=0, numChoices=0, compteur=0, theChosen=0;

  for (i=0; i<NUM_BONUS; i++) {
    if (hasardTable[i]) numChoices++;
  }

#ifdef WIN32
  theChoice = rand() % numChoices +1;
#else
  theChoice = random() % numChoices +1;
#endif

  for (i=0; i < NUM_BONUS; i++) {
    if (hasardTable[i]) compteur++;
    if (compteur == theChoice) {theChosen=i ; i=NUM_BONUS;}
  }

  printf("...and the winner is... bonus %d!\n",theChosen);
  if ((theChosen > 0)&&(world->current_bonus > -1)) 
    world->tab_bonus[world->current_bonus]->cancelAction();
  world->tab_bonus[theChosen]->agir(parametre);
}

void B_HASARD::construit(GLint typeListe)
{
  liste = typeListe;
  LoadTexture(textureBonus[E_HASARD], &texture);
}

void B_HASARD::annuleEffets()
{
}

void B_NUIT::agir(int parametre)
{
  printf("Good Night!\n");
  actif=1;
  //player->score += 20;
  for (int i=0; i<4; i++)
    world->mat_amb_diff[i]-=0.85;
  setUpTimer(parametre);
}

void B_NUIT::construit(GLint typeListe)
{
  liste = typeListe;
  LoadTexture(textureBonus[E_NUIT], &texture);
  timer = new CHRONOMETRE;
}

void B_NUIT::annuleEffets()
{
  for (int i=0; i<4; i++)
    world->mat_amb_diff[i]+=0.85;
}

void B_GLACON::agir(int parametre)
{
  int i=0;
  actif=1;
  player->score += 20;
  for (i=0; i<world->num_huts; i++) 
    change_hut_status(&world->huts[i],ETAT_FREEZE);

  for (i=0; i<4; i++)
    if (i!=2) world->mat_amb_diff[i]-=0.55;

  setUpTimer(parametre);
}

void B_GLACON::construit(GLint typeListe)
{
  liste = typeListe;
  LoadTexture(textureBonus[E_GLACON], &texture);
  timer = new CHRONOMETRE;
}

void B_GLACON::annuleEffets()
{
  int i=0;
  for (i=0; i< world->num_huts; i++) 
    change_hut_status(&world->huts[i],ETAT_FONCTIONNE);

  for (i=0; i<4; i++)
    if (i!=2) world->mat_amb_diff[i]+=0.55;
}

void B_COUPDEBOL::agir(int parametre)
{
  int theChoice=0;

#ifdef WIN32
  theChoice = rand() % 9;
#else
  theChoice = random() % 9;
#endif

  printf("Lucky man ! We give you bonus %d.\n",theChoice);
  world->tab_bonus[theChoice]->agir(parametre);

}

void B_COUPDEBOL::construit(GLint typeListe)
{
  liste = typeListe;
  LoadTexture(textureBonus[E_COUPDEBOL], &texture);
}

void B_COUPDEBOL::annuleEffets()
{
}

void B_ONDEDECHOC::agir(int parametre)
{
  player->score += 20;
  for (int i=0; i < world->num_huts; i++)
    {
      shockWave(&world->huts[i],parametre);
    }
}

void B_ONDEDECHOC::construit(GLint typeListe)
{
  liste = typeListe;
  LoadTexture(textureBonus[E_ONDEDECHOC], &texture);
}

void B_ONDEDECHOC::annuleEffets()
{
}

void B_SHMIXGOMME::agir(int parametre)
{
  actif=1;
  player->score += 100;
  player->status = ETAT_POURSUIVANT;
  setUpTimer(parametre);
}

void B_SHMIXGOMME::construit(GLint typeListe)
{
  liste = typeListe;
  LoadTexture(textureBonus[E_SHMIXGOMME], &texture);
  timer = new CHRONOMETRE;
}

void B_SHMIXGOMME::annuleEffets()
{
  player->status = ETAT_POURSUIVI;
}

void B_BOUCLIER::agir(int parametre)
{
  actif=1;
  player->score += 10;
  player->highlander=1;
  setUpTimer(parametre);
}

void B_BOUCLIER::construit(GLint typeListe)
{
  liste = typeListe;
  LoadTexture(textureBonus[E_BOUCLIER], &texture);
  timer = new CHRONOMETRE;
}

void B_BOUCLIER::annuleEffets()
{
  player->highlander=0;
}

void B_PERDBONUS::agir(int parametre)
{
  player->score += 10;
  for (int i=0; i<4; i++)
    {
      player->stockBonus[i]=0;
    }
}

void B_PERDBONUS::construit(GLint typeListe)
{
  liste = typeListe;
  LoadTexture(textureBonus[E_PERDBONUS], &texture);
}

void B_PERDBONUS::annuleEffets()
{
}

void B_ARMAGGEDON::agir(int parametre)
{
  all_shmol_go_home();
  hut_t *hut;
  player->score += 10;
  for (int i=0; i<world->num_huts; i++)
    {
      hut = &world->huts[i];
      //change_shmol_status(hut, ETAT_MORT);
      if (player->square == hut->square) {
	change_hut_status(hut, ETAT_DETRUIT);
	printf("Shmollux's home (square %d) destroyed !\n", hut->square);
      }
    }
}

void B_ARMAGGEDON::construit(GLint typeListe)
{
  liste = typeListe;
  LoadTexture(textureBonus[E_ARMAGGEDON], &texture);
}

void B_ARMAGGEDON::annuleEffets()
{
}

void B_RAPIDE::agir(int parametre)
{
  actif=1;
  player->score += 10;
  player->rotat_speed[0]=VITESSE_ROTAT_MARCHE*2; 
  player->rotat_speed[1]=VITESSE_ROTAT_COURSE*2;
  player->trans_speed[0]=VITESSE_TRANS_MARCHE*2; 
  player->trans_speed[1]=VITESSE_TRANS_COURSE*2;

  setUpTimer(parametre);
}

void B_RAPIDE::construit(GLint typeListe)
{
  liste = typeListe;
  LoadTexture(textureBonus[E_RAPIDE], &texture);
  timer = new CHRONOMETRE;
}

void B_RAPIDE::annuleEffets()
{
  player->rotat_speed[0]=VITESSE_ROTAT_MARCHE; 
  player->rotat_speed[1]=VITESSE_ROTAT_COURSE;
  player->trans_speed[0]=VITESSE_TRANS_MARCHE; 
  player->trans_speed[1]=VITESSE_TRANS_COURSE;
}

void B_LENT::agir(int parametre)
{
  printf("Loooow speed\n");
  actif=1;
  player->score += 10;
  player->rotat_speed[0]=VITESSE_ROTAT_MARCHE/2; 
  player->rotat_speed[1]=VITESSE_ROTAT_MARCHE/2;
  player->trans_speed[0]=VITESSE_TRANS_MARCHE/2; 
  player->trans_speed[1]=VITESSE_TRANS_MARCHE/2;
  setUpTimer(parametre);
}

void B_LENT::construit(GLint typeListe)
{
  liste = typeListe;
  LoadTexture(textureBonus[E_LENT], &texture);
  timer = new CHRONOMETRE;
}

void B_LENT::annuleEffets()
{
  player->rotat_speed[0]=VITESSE_ROTAT_MARCHE; 
  player->rotat_speed[1]=VITESSE_ROTAT_COURSE;
  player->trans_speed[0]=VITESSE_TRANS_MARCHE; 
  player->trans_speed[1]=VITESSE_TRANS_COURSE;
}

void B_STOPTEMPS::agir(int parametre)
{
  actif=1;
  player->score += 10;
  world->game_timer->togglePaused();
  setUpTimer(parametre);
}

void B_STOPTEMPS::construit(GLint typeListe)
{
  liste = typeListe;
  LoadTexture(textureBonus[E_STOPTEMPS], &texture);
  timer = new CHRONOMETRE;
}

void B_STOPTEMPS::annuleEffets()
{
  world->game_timer->togglePaused();
}

void B_ULTRAPOINTS::agir(int parametre)
{
  actif=1;
  player->score += 10;
  world->dituboite->valeur=10;
  setUpTimer(parametre);
}

void B_ULTRAPOINTS::construit(GLint typeListe)
{
  liste = typeListe;
  LoadTexture(textureBonus[E_ULTRAPOINTS], &texture);
  timer = new CHRONOMETRE;
}

void B_ULTRAPOINTS::annuleEffets()
{
  world->dituboite->valeur=1;
}

void B_INVERTTOUCHES::agir(int parametre)
{
  actif=1;
  //player->score += 10;
  player->theKeys[KEY_UP] = SDLK_DOWN;
  player->theKeys[KEY_DOWN] = SDLK_UP;
  player->theKeys[KEY_LEFT] = SDLK_RIGHT;
  player->theKeys[KEY_RIGHT] = SDLK_LEFT;
  setUpTimer(parametre);
}

void B_INVERTTOUCHES::construit(GLint typeListe)
{
  liste = typeListe;
  LoadTexture(textureBonus[E_INVERTTOUCHES], &texture);
  timer = new CHRONOMETRE;
}

void B_INVERTTOUCHES::annuleEffets()
{
  player->theKeys[KEY_UP] = SDLK_UP;
  player->theKeys[KEY_DOWN] = SDLK_DOWN;
  player->theKeys[KEY_LEFT] = SDLK_LEFT;
  player->theKeys[KEY_RIGHT] = SDLK_RIGHT;
}

void B_PLAFOND::agir(int parametre)
{
  actif=1;
  player->elevation = AU_PLAFOND;
  //player->score += 10;
  setUpTimer(parametre);
}

void B_PLAFOND::construit(GLint typeListe)
{
  liste = typeListe;
  LoadTexture(textureBonus[E_PLAFOND], &texture);
  timer = new CHRONOMETRE;
}

void B_PLAFOND::annuleEffets()
{
  player->elevation = AU_SOL;
}

void B_FOG::agir(int parametre)
{
  actif=1;
  
  GLfloat color[4]={0.5f,0.5f,0.5f,1.0f};
  glFogfv(GL_FOG_COLOR, color);
  glFogf(GL_FOG_START, 0.0);
  glFogf(GL_FOG_END, 10.0);
  glFogi(GL_FOG_MODE, GL_LINEAR);
  glFogf(GL_FOG_DENSITY, 1.0);
  glClearColor(0.5f,0.5f,0.5f,1.0f);
  glEnable(GL_FOG);

  glEnable(GL_FOG);
  setUpTimer(parametre);
  //player->score += 10;
}

void B_FOG::construit(GLint typeListe)
{
  liste = typeListe;
  LoadTexture(textureBonus[E_FOG], &texture);
  timer = new CHRONOMETRE;
}

void B_FOG::annuleEffets()
{
  GLfloat color[4]={0.0f,0.0f,0.0f,1.0f};
  glFogfv(GL_FOG_COLOR, color);
  glFogf(GL_FOG_START, 15.0);
  glFogf(GL_FOG_END, 40.0);
  //glFogf(GL_FOG_MODE, GL_LINEAR);
  glFogf(GL_FOG_DENSITY, 0.1);
  glClearColor(0.0f,0.0f,0.0f,0.0f);
  glEnable(GL_FOG);

//  glDisable(GL_FOG);
}

void B_MALUSMAP::agir(int parametre)
{
  actif=1;
  //player->score += 10;
  player->sees_minimap=0;
  setUpTimer(parametre);
}

void B_MALUSMAP::construit(GLint typeListe)
{
  liste = typeListe;
  LoadTexture(textureBonus[E_MALUSMAP], &texture);
  timer = new CHRONOMETRE;
}

void B_MALUSMAP::annuleEffets()
{
  player->sees_minimap=1;
}

void B_ANTIBOITES::agir(int parametre)
{
  actif=1;
  //player->score += 10;
  world->dituboite->valeur = -world->dituboite->valeur;
  setUpTimer(parametre);
}

void B_ANTIBOITES::construit(GLint typeListe)
{
  liste = typeListe;
  LoadTexture(textureBonus[E_ANTIBOITES], &texture);
  timer = new CHRONOMETRE;
}

void B_ANTIBOITES::annuleEffets()
{
  world->dituboite->valeur = -world->dituboite->valeur;
}

void B_FREEZESHX::agir(int parametre)
{
  player->score += 20;
  actif=1;
  //player->rotat_speed[0]=0; player->rotat_speed[1]=0;
  player->trans_speed[0]=0; player->trans_speed[1]=0;
  setUpTimer(parametre);

}

void B_FREEZESHX::construit(GLint typeListe)
{
  liste = typeListe;
  LoadTexture(textureBonus[E_FREEZESHX], &texture);
  timer = new CHRONOMETRE;
}

void B_FREEZESHX::annuleEffets()
{
  //player->rotat_speed[0]=VITESSE_ROTAT_MARCHE; 
  //player->rotat_speed[1]=VITESSE_ROTAT_COURSE;
  player->trans_speed[0]=VITESSE_TRANS_MARCHE; 
  player->trans_speed[1]=VITESSE_TRANS_COURSE;
}
