/*  The Blue Mango Quest
 *  Copyright (c) Clment 'phneutre' Bourdarias (code)
 *                   email: phneutre@users.sourceforge.net
 *                Guillaume 'GuBuG' Burlet (graphics)
 *                   email: gubug@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef WIN32
#include <windows.h>
#endif
#include <GL/gl.h>
#include <GL/glu.h>
#include <SDL/SDL.h>

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "world_building.h"
#include "map.h"
#include "timers.h"
#include "world_geometry.h"
#include "hut.h"
#include "bonus.h"
#include "mango.h"
#include "game_loop.h"
#include "system_gl.h"
#include "draw_scene_gl.h"
#include "sector.h"
#include "move.h"

extern player_t *player;
extern game_data_t *world;
char PAUSE = 0;

void new_round()
{
  reset_player_pos();
  all_shmol_go_home();
  player->status = STATUS_WAITING;
  world->mat_amb_diff[0] = 0.937;
  world->mat_amb_diff[1]= 0.937;
  world->mat_amb_diff[2] = 0.937;
  world->mat_amb_diff[3] = 1.0;
}

void special_loop(int &game_end, char status)
{
  int loop_end=0;
  CHRONOMETRE *loop_timer=0;

//  world->game_sync->update_temps();
//  world->game_timer->update_temps();

  if (status == STATUS_DEAD)
    {
      loop_timer = new CHRONOMETRE;
      loop_timer->initialise();
      loop_timer->setAlarm(2);
    }
//  world->game_sync->update_temps();
//  world->game_timer->update_temps();

  player->mySector = get_sector(player->pos_x, player->pos_z);
  if (world->current_bonus > -1) 
    world->tab_bonus[world->current_bonus]->cancelAction();
  world->current_bonus = -1 ;

  while(!(loop_end))
    {
      world->game_sync->update_temps();
      world->game_timer->update_temps();
      if (status == STATUS_DEAD)
	{
	  if (loop_timer->update_temps()) loop_end=1;
	}

  {
    SDL_Event event;

    while (SDL_PollEvent(&event))
    {
      if (event.type == SDL_KEYDOWN)
      {
        switch (event.key.keysym.sym)
        {
        case SDLK_SPACE:
	  if (status == GAME_OVER) game_end=1;
          loop_end = 1;
          break;

	case SDLK_ESCAPE:
          if ((status != STATUS_DEAD)
	      || (status != STATUS_PAUSE)) game_end = 1;

	  loop_end = 1;
          break;
        
	default:
          break;
        }
      }
    }
  }
  
  draw_entire_scene_GL();
    }
  
  switch (status)
    {
    case STATUS_WAITING:
      player->status = ETAT_POURSUIVI;
      break;

    case STATUS_DEAD:
      new_round();
      break;

    case STATUS_WIN:
    case STATUS_PAUSE:
      break;
    }

  if (status == STATUS_DEAD) delete loop_timer;
}

int mainLoop()
{
  int gameEnd = 0;

  world->game_sync->initialise();

  world->game_timer->initialise();
  world->game_timer->setAlarm(world->time_limit);

  printf ("\n-- STEP 3: Falling into the main game loop --\n");
  player->mySector = get_sector(player->pos_x, player->pos_z);

  while (!(gameEnd))
  {
    if ((player->status == STATUS_WAITING) || (player->status == STATUS_WIN)
	|| (player->status == STATUS_DEAD) || (player->status == GAME_OVER))
      {
	world->game_timer->togglePaused();
	special_loop(gameEnd, player->status);
	world->game_timer->togglePaused();
    }

    if (PAUSE == 1)
      {
	world->game_timer->togglePaused();
	special_loop(gameEnd, STATUS_PAUSE);
	PAUSE = 0;
	world->game_timer->togglePaused();
      }

    world->game_sync->update_temps();
    if ((world->game_timer->update_temps())&&(world->time_limit)) 
      game_over();

    update_input(gameEnd);
    player->mySector = get_sector(player->pos_x, player->pos_z);
    if (!(player->mySector)) 
      {
	fprintf(stderr, "Error: Not In A Sector (this is a known bug).\n");
	exit(1);
      }

    update_special();
    update_ia();
    update_bonus();

    draw_entire_scene_GL();
  }
  return 1;
}

void update_input(int &fin)
{

  Uint8 *keys;
  SDLMod modifs;

////////////////////////////////////////////            
  {
    SDL_Event event;

    while (SDL_PollEvent(&event))
    {
      if (event.type == SDL_KEYDOWN)
      {

        switch (event.key.keysym.sym)
        {
        case SDLK_ESCAPE:
          fin = 1;
          break;
	case SDLK_F1:
	  if ((player->stockBonus[0])&&(whichBonusActive()<= E_COUPDEBOL))
	    {
	      if (world->current_bonus > -1) 
		world->tab_bonus[world->current_bonus]->cancelAction();
	      world->tab_bonus[E_ARMAGGEDON]->agir(player->paramStocks[0]);
	      player->stockBonus[0]=0;
	    }
          break;

        case SDLK_F2:
	  if ((player->stockBonus[1])&&(whichBonusActive()<=E_COUPDEBOL))
	    {
	      if (world->current_bonus > -1) 
		world->tab_bonus[world->current_bonus]->cancelAction();
	      world->tab_bonus[E_ONDEDECHOC]->agir(player->paramStocks[1]);
	      player->stockBonus[1]=0;
	    }
          break;
        case SDLK_F3:
	  if ((player->stockBonus[2])&&(whichBonusActive()<=E_COUPDEBOL))
	    {
	      if (world->current_bonus > -1) 
		world->tab_bonus[world->current_bonus]->cancelAction();
	      world->tab_bonus[E_RAPIDE]->agir(player->paramStocks[2]);
	      player->stockBonus[2]=0;
	      world->current_bonus = E_RAPIDE;
	    }
          break;
        case SDLK_F4:
	  if ((player->stockBonus[3])&&(whichBonusActive()<=E_COUPDEBOL))
	    {
	      if (world->current_bonus > -1) 
		world->tab_bonus[world->current_bonus]->cancelAction();
	      world->tab_bonus[E_STOPTEMPS]->agir(player->paramStocks[3]);
	      player->stockBonus[3]=0;
	      world->current_bonus = E_STOPTEMPS;
	    }
          break;

	case SDLK_p:
	  PAUSE = 1;
	  break;


        default:
          break;
        }
      }
    }
  }
////////////////////////////////////////////    

  keys = SDL_GetKeyState(NULL);
  modifs = SDL_GetModState();

  if (modifs & KMOD_SHIFT)
  {
    if (!player->run)
      player->run = 1;
  }
  else if (player->run)
    player->run = 0;

  if (modifs & KMOD_ALT)
  {
    if (!player->slide)
      player->slide = 1;
  }
  else if (player->slide)
    player->slide = 0;

  if (keys[player->theKeys[KEY_UP]] == SDL_PRESSED)
  {
    move_forward();
  }

  if (keys[player->theKeys[KEY_DOWN]] == SDL_PRESSED)
  {
    move_backward();
  }

  if (((player->slide == SHX_LATERAL) || (player->slide == SHX_DROITE))
      && (keys[player->theKeys[KEY_RIGHT]] == SDL_PRESSED))
  {
    move_slide_right();
    if (keys[player->theKeys[KEY_LEFT]] == SDL_PRESSED)
      rotate(SHX_GAUCHE);
  }

  if ((keys[player->theKeys[KEY_RIGHT]] != SDL_PRESSED) && (player->slide))
    player->slide = SHX_LATERAL;

  if (((player->slide == SHX_LATERAL) || (player->slide == SHX_GAUCHE))
      && (keys[player->theKeys[KEY_LEFT]] == SDL_PRESSED))
  {
    move_slide_left();
    if (keys[player->theKeys[KEY_RIGHT]] == SDL_PRESSED)
      rotate(SHX_DROITE);
  }

  if ((keys[player->theKeys[KEY_LEFT]] != SDL_PRESSED) && (player->slide))
    player->slide = SHX_LATERAL;

  if ((keys[player->theKeys[KEY_RIGHT]] == SDL_PRESSED) && (!(player->slide)))
  {
    rotate(SHX_DROITE);
  }

  if ((keys[player->theKeys[KEY_LEFT]] == SDL_PRESSED) && (!(player->slide)))
  {
    rotate(SHX_GAUCHE);
  }

  if (keys[SDLK_PAGEUP] == SDL_PRESSED)
  {
    look_around(SHX_HAUT);
  }

  if (keys[SDLK_PAGEDOWN] == SDL_PRESSED)
  {
    look_around(SHX_BAS);
  }

  if (keys[SDLK_END] == SDL_PRESSED)
  {
    look_around(SHX_CENTRE);
  }

}

void update_special()
{
  int typeSpecial=0;
  if (player->elevation == AU_SOL)
    {
      typeSpecial = test_special(player->square);

      if ((typeSpecial >= TEL_D1)&&(typeSpecial <= TEL_A5))
	{
	  if (player->teleporting < 0) {

	    player->teleporting = world->map[player->square].paramSpecial ;
	    teleport_to(world->map[player->square].paramSpecial);
	  }
	}

      if (player->teleporting) {
	if (player->square != player->teleporting) 
	  player->teleporting = -1;
      }
    }
}

void update_ia()
{
   hut_t *hut;
   for (int i=0; i < world->num_huts; i++)
     {
       hut = &world->huts[i];
       update_shmollux(hut);
     }
}

void update_bonus()
{
  int param=0, bonus_type = 0;
  world->bonus_rotat += 0.1*world->game_sync->dt;
    if (world->bonus_rotat >= 360)
      world->bonus_rotat = 0;

    world->current_bonus = whichBonusActive();

  if (world->current_bonus >= 0) {
    
    // bonus is no longer active if updateTime returns 0
    if (world->tab_bonus[world->current_bonus]->updateTimer()==0)
      world->current_bonus = -1;
  }

  if (world->tab_bonus[E_SHMIXGOMME]->actif) {
	world->b_shmixgomme->updateTimer();
  }

  if (player->elevation == AU_SOL)
    {

      bonus_type = test_bonus(player->square);

      switch (bonus_type)
	{
	case E_RIEN:
	  break;
	case E_DITUBOITE:
	  world->map[player->square].is_ditugomme = 0;
	  world->num_dituboites--;
	  if (!(desactive_bonus(player->mySector, player->square)))
	    fprintf(stderr,"Error: can't find bonus on square %d\n",
		    player->square);
	  world->dituboite->agir(param);

	  if (world->num_dituboites == 0)
	    player->status = STATUS_WIN;
	  break;

	case E_ARMAGGEDON:
	  stock_bonus(0);
	  break;

	case E_ONDEDECHOC:
	  stock_bonus(1);
	  break;

	case E_RAPIDE:
	  stock_bonus(2);
	  break;

	case E_STOPTEMPS:
	  stock_bonus(3);
	  break;

	case E_PERDBONUS:
	case E_SHMIXGOMME:
	case E_HASARD:
	  execute_bonus(bonus_type);
	  break;

	default:
	  if (world->current_bonus <= E_COUPDEBOL) // a good bonus is active
	    {
	      if (world->current_bonus > -1) 
		world->tab_bonus[world->current_bonus]->cancelAction();
	      execute_bonus(bonus_type);
	    }

	  else  // a malus is active, we wait for it to finish 
	    {


	    }
   
	  break;
	}
    }

}

void toggle(int what)
{
  int check = 0;

  if (!(what))
  {
    what = 1;
    check = 1;
  }

  if ((what) && (!(check)))
  {
    what = 0;
  }

  //what = !what;
}

void game_over()
{
  player->status = GAME_OVER;
  printf("You are a loser. Game is over.\n");
}
