/*  The Blue Mango Quest
 *  Copyright (c) Clment 'phneutre' Bourdarias (code)
 *                   email: phneutre@users.sourceforge.net
 *                Guillaume 'GuBuG' Burlet (graphics)
 *                   email: gubug@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef WIN32
#include <windows.h>
#endif
#include <GL/gl.h>
#include <GL/glu.h>
#include <SDL/SDL.h>
#include <SDL/SDL_mixer.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>

#include "world_geometry.h"
#include "world_building.h"
#include "map.h"
#include "timers.h"
#include "hut.h"
#include "bonus.h"
#include "mango.h"
#include "game_loop.h"
#include "system_gl.h"

#ifndef WIN32
#   include <unistd.h>
#   include <sys/types.h>
#   include <pwd.h>
#   include <dirent.h>
#   include <sys/time.h>
#   include <sys/types.h>
#   include <dirent.h>
#   include <errno.h>
#endif

#include "file_utils.h"

char *filename = 0;

void clean_exit(void);
void initialize_system();
void analyze_args();

void usage_error(void)
{
  fprintf(stderr, "Usage:  shmixman3d [file] ..\n\n");
  /* fprintf(stderr,"Options:\n" " -w force world (0/1/2/3)\n\n"); */
  exit(1);
}

#define GET_NUMBER(flag,num)	num = 0; \
				if (!*p && argc && isdigit(**argv)) \
					p = *argv++, --argc; \
				while (isdigit(*p)) { \
					flag = 1; \
					num = (num * 10) + (*p++ - '0'); \
				}

void analyze_args(int argc, char *argv[])
{
  char c, *p;

  ++argv;

  if (!--argc)
  {
    // Aucun argument donn 
    filename = new char[MAX_PATH];
    if (search_map_file(filename, "default.shx", MAX_PATH))
      {
	fprintf(stderr, "Error: couldn't find map.\n");
      }
  }

  while (argc--)
  {
    p = *argv++;
    if (*p == '-')
    {
      if (!*++p)                // argument incomplet
        usage_error();
      while ((c = *p++))
      {
        switch (c)
        {

        case 'v':
          fprintf(stdout, "The Blue Mango Quest, version %s\n", VERSION);
          exit(0);
          break;

        case 'h':

        default:
          usage_error();
        }
      }
      if (!argc)
      {
        // Aucun nom du fichier donn
        filename = new char[MAX_PATH];

	if (search_map_file(filename, "default.shx", MAX_PATH)) 
	  {
	    fprintf(stderr,"Error: couldn't find map\n");
	    exit(1);
	  }
	printf("Using default map %s.\n", filename);
        
      }

    }
    else
    {
      //map_name = p;
      filename = new char[MAX_PATH];
      if (search_map_file(filename, p, MAX_PATH)) 
	{
	  fprintf(stderr,"Error: couldn't find map\n");
	  exit(1);
	}
    }
  }
}

int main(int argc, char *argv[])
{

#ifdef WIN32
  if (!getenv("SDL_VIDEODRIVER"))
    putenv("SDL_VIDEODRIVER=windib");
#endif

  atexit(clean_exit);
  printf("\n-- The Blue Mango Quest version %s (%s, %s) --\n", VERSION,
         __DATE__, __TIME__);
  printf
    ("(c) 2000-2001 Clment Bourdarias (code), Guillaume Burlet (graphics)\n\n");

  check_private_dir();
  analyze_args(argc, argv);
  initialize_system();

  register_world();

  printf("\n-- STEP 1: Reading map file --\n");
  load_map(filename);
  build_world();
  init_characters();

  init_GL();
 

  mainLoop();

  cleanup_world();
  return 0;
}

void clean_exit()
{
  fprintf(stderr, "Exiting via clean_exit()\n");
  SDL_Quit();
  printf("\n");
}

void initialize_system()
{
  printf("Initializing sound, video and timer... ");

  if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_CDROM | SDL_INIT_TIMER) < 0)
  {
    fprintf(stderr, "\nFatal Error : %s\n", SDL_GetError());
    exit(MAX_PATH);
  }

  SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
  SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 16);

#ifdef WIN32
  if (SDL_SetVideoMode
      (800, 600, 16, SDL_OPENGL | SDL_FULLSCREEN | SDL_DOUBLEBUF
		) == NULL)
#else
  if (SDL_SetVideoMode(800, 600, 16, SDL_OPENGL | SDL_GL_DOUBLEBUFFER) == NULL)
#endif
  {
    fprintf(stderr, "\nError (OpenGL) : %s\n", SDL_GetError());
    SDL_Quit();
    exit(2);
  }

  SDL_WM_SetCaption("The Blue Mango Quest v." VERSION, NULL);

#ifdef WIN32
  srand(time(NULL));
#else
  srandom(time(NULL));
#endif

  printf("done!\n");
}
